//------------------------------------
float4x4 worldViewProjMat;				// World View Projection Matrix
float4x4 worldViewIT;					// InverseTransformed ViewProjection Matrix
//float4 zVec;

texture2D dataTexture;					//DataTexture
texture2D velocityDataTexture;			//VelocityDataTexture;
texture1D tfDataTexture;				//TransferfunctionDataTexture
texture1D tfArrowTexture;				//TransferfunctionArrowTexture
int backDataType;

float arrowXPos;
float arrowYPos;

int arrowColorMethod;
int arrowSizeMethod;

float sU;
float sV;
float eU;
float eV;

//------------------------------------
/**
Vertex Input
*/
struct vertexInput {
    float3 position				: POSITION;
    float4 texCoord				: TEXCOORD0;
};

/**
Vertex Output
*/
struct vertexOutput {
	float4 position				: POSITION;
	float4 texCoord				: TEXCOORD0;
};

/**
Vertex Input
*/
struct vertexAInput {
    float3 position				: POSITION;
    //float4 texCoord				: TEXCOORD0;
};

/**
Vertex Output
*/
struct vertexAOutput {
	float4 position				: POSITION;
	//float4 texCoord				: TEXCOORD0;
};

/**
Pixel Ouput
*/
struct pixelOutput
{
	float4 color				: COLOR;
};

//------------------------------------
/**
Linear sampler template
*/
#define SAMPLER_LINEAR(g_samplerMap, g_txMap);	\
	sampler2D g_samplerMap = sampler_state {	\
    Texture = <g_txMap>;						\
    MinFilter = Linear;							\
    MagFilter = Linear;							\
    MipFilter = Linear;							\
    AddressU  = CLAMP;							\
    AddressV  = CLAMP;							\
};

/**
Point sampler template
*/
#define SAMPLER_POINT(g_samplerMap, g_txMap);	\
	sampler2D g_samplerMap = sampler_state {	\
    Texture = <g_txMap>;						\
    MinFilter = Point;							\
    MagFilter = Point;							\
    MipFilter = Point;							\
    AddressU  = BORDER;							\
    AddressV  = BORDER;							\
};

/**
Sampler for the transferFunction
*/
sampler1D samplerTFData = sampler_state
{
	Texture = <tfDataTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
    AddressU = CLAMP;
    AddressV = CLAMP;
};

sampler1D samplerTFArrow = sampler_state
{
	Texture = <tfArrowTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
    AddressU = CLAMP;
    AddressV = CLAMP;
};


/**
Linear Sampler for frontFaceTexture
*/
SAMPLER_LINEAR(samplerData, dataTexture);
SAMPLER_LINEAR(samplerVelocityData, velocityDataTexture);

//----------------------------------------------


vertexOutput BackgroundVS(vertexInput IN) 
{
    vertexOutput OUT;
	float4 vertexpos = float4(IN.position.xyz, 1.0);
	float4 eyespace = mul(vertexpos, worldViewProjMat);
	OUT.position = eyespace;
	OUT.texCoord = float4(IN.texCoord.xyz, 0);
	return OUT;
}

/**
Pixelshader for slices
*/
pixelOutput BackgroundPS(vertexOutput IN) 
{
	pixelOutput OUT;
	float value = 0;
	//float2 v;
	
	if(backDataType == 0) {
		value = tex2D(samplerData, IN.texCoord.xy).r;
		//v = (v-0.5f)*2.0f;
		//value = sqrt(v.x*v.x + v.y*v.y);
	} else if(backDataType == 1) {
		value = tex2D(samplerData, IN.texCoord.xy).g;
	} else {
		value = tex2D(samplerData, IN.texCoord.xy).b;
	}
	OUT.color = tex1D(samplerTFData, value);
	/*if(IN.texCoord.x>sU && IN.texCoord.x< eU &&
		IN.texCoord.y>sV && IN.texCoord.y< eV) {
			OUT.color = float4(1, 0 , 0 ,0);
		} else {
		OUT.color = float4(0, 0 , 1 ,0);
		}*/
	return OUT;
}

vertexAOutput ArrowVS(vertexAInput IN) 
{
    vertexAOutput OUT;
	float4 vertexpos = float4(IN.position.xyz, 1.0);
	float4 eyespace = mul(vertexpos, worldViewProjMat);
	OUT.position = eyespace;
	return OUT;
}

/**
Pixelshader for slices
*/
pixelOutput ArrowPS(vertexAOutput IN) 
{
	pixelOutput OUT;
	float value = 0;
	//float2 v;
	if(arrowColorMethod==0) {			//Constant
		value = 1.0f;
	} else if(arrowColorMethod==1) {	//Velocity
		value = tex2D(samplerData, float2(arrowXPos, arrowYPos)).r;
		//v = tex2D(samplerData, float2(arrowXPos, arrowYPos)).ra;
		//v = (v-0.5f)*2.0f;
		//value = sqrt(v.x*v.x + v.y*v.y);
	} else if(arrowColorMethod==2) {	//Pressure
		value = tex2D(samplerData, float2(arrowXPos, arrowYPos)).g;
	} else {							//Vorticity
		value = tex2D(samplerData, float2(arrowXPos, arrowYPos)).b;
	}
	OUT.color = tex1D(samplerTFArrow, value);
	return OUT;
}

// 
// Effect
//

#define Technique(name);								\
	technique name										\
	{													\
	    pass p0											\
	    {												\
		    VertexShader = compile vs_3_0 name##VS();	\
		    PixelShader  = compile ps_3_0 name##PS();	\
		}												\
	}

Technique( Background );
Technique( Arrow );